'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.EventListener = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _tslib = require('tslib');

var tslib_1 = _interopRequireWildcard(_tslib);

var _react = require('react');

var _pure = require('../../utils/pure');

var _object = require('@devexperts/utils/lib/object');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var ON_MARKER = 'on';
var CAPTURE_MARKER = 'Capture';
var EventListener = function (_Component) {
    _inherits(EventListener, _Component);

    function EventListener() {
        _classCallCheck(this, EventListener);

        return _possibleConstructorReturn(this, (EventListener.__proto__ || Object.getPrototypeOf(EventListener)).apply(this, arguments));
    }

    _createClass(EventListener, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
            this.addListeners();
        }
    }, {
        key: 'componentWillUpdate',
        value: function componentWillUpdate(nextProps) {
            if (shouldResetListeners(this.props, nextProps)) {
                this.removeListeners();
            }
        }
    }, {
        key: 'componentDidUpdate',
        value: function componentDidUpdate(prevProps) {
            if (shouldResetListeners(prevProps, this.props)) {
                this.addListeners();
            }
        }
    }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
            this.removeListeners();
        }
    }, {
        key: 'render',
        value: function render() {
            return this.props.children;
        }
    }, {
        key: 'addListeners',
        value: function addListeners() {
            var target = this.getTarget();
            var handlers = this.getHandlers();
            Object.keys(handlers).forEach(function (key) {
                var capture = key.endsWith(CAPTURE_MARKER);
                var handler = handlers[key];
                var eventName = getEventName(key, capture);
                target.addEventListener(eventName, handler, capture);
            });
        }
    }, {
        key: 'removeListeners',
        value: function removeListeners() {
            var target = this.getTarget();
            var handlers = this.getHandlers();
            Object.keys(handlers).forEach(function (key) {
                var capture = key.endsWith(CAPTURE_MARKER);
                var handler = handlers[key];
                var eventName = getEventName(key, capture);
                target.removeEventListener(eventName, handler, capture);
            });
        }
    }, {
        key: 'getTarget',
        value: function getTarget() {
            var target = this.props.target;

            if (typeof target === 'string') {
                return window[target];
            }
            return target;
        }
    }, {
        key: 'getHandlers',
        value: function getHandlers() {
            // noinspection JSUnusedLocalSymbols
            var _a = this.props,
                target = _a.target,
                children = _a.children,
                props = tslib_1.__rest(_a, ["target", "children"]);
            var propKeys = Object.keys(props);
            var eventKeys = propKeys.filter(function (key) {
                return key.startsWith('on');
            });
            var handlers = eventKeys.reduce(function (acc, key) {
                acc[key] = props[key];
                return acc;
            }, {});
            return handlers;
        }
    }]);

    return EventListener;
}(_react.Component);
exports.EventListener = EventListener = tslib_1.__decorate([_pure.PURE], EventListener);
exports.EventListener = EventListener;

function getEventName(rawEventName, capture) {
    var trimmedLeft = rawEventName.startsWith(ON_MARKER) ? rawEventName.substring(ON_MARKER.length) : rawEventName;
    var trimmed = capture ? trimmedLeft.substring(0, trimmedLeft.length - CAPTURE_MARKER.length) : trimmedLeft;
    return trimmed.toLowerCase();
}
function shouldResetListeners(prevProps, nextProps) {
    var prevChildren = prevProps.children,
        oldProps = tslib_1.__rest(prevProps, ["children"]);var nextChildren = nextProps.children,
        newProps = tslib_1.__rest(nextProps, ["children"]);
    return !(0, _object.shallowEqual)(oldProps, newProps);
}